/* $Workfile: 2xVd5x16.c $       $Revision: 7 $      $Modtime: 11/02/99 1:51p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc PublicDemo

  @module

    2xVidar5x16.c

    An example console application demonstrating the use of OTX Driver
    with 2 Vidar-5x16-2-PCI boards connected over the CT Bus.

    The sample application provides a simple console interface which utilizes
    simple keystrokes as commands.

    Handsets 0 and 1 are through connected in the time-space switch on each board.

    Handsets 2 is through connected over the H.100 bus (Highway0:TS0 and Highway0:TS1)
    to Handset 2 on the other board. This connection is established when the user
    presses 'c'. This connection is disconnected when the user presses 'd'.

    Press 'h' or '?' for help. 'q' to quit.

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>


#include "OsaStd.h"
#include "OsaCon.h"
#include "OsaThrd.h"
#include "OtxErr.h"
#include "OtxBrd.h"
#include "OtxDef.h"
#include "OtxType.h"
#include "OtxDrv.h"
#include "OtxData.h"

#define DEMO_DEFAULT_LOG_FILE "2xVidar5x16.log"
#define DEMO_NO_OF_BRDS 2  // Number of Vidar-5x16-PCI boards used by this application
#define DEMO_MAX_CODECS 4  // Maximum number of codecs supported

// This example program uses a fixed connection method by opening the Time-Space Switch (TSS)
// physical device. Pipes connected to the TSS can be viewed as a pipe within the TSS device
// itself. The TSS device has 16 local highways, and 32 H.100 highways. To make a connection
// between a device (e.g. Codec) on a local highway and one H.100 highway, a pipe is first
// created, then connected using the handle to the TSS physical device.
//
// Future releases of the driver will have a simplified way of connecting between
// two OTX boards where the actual highway number and timeslots used by the TSS will not
// have to be known by the application writer.
#define DEMO_CODEC_HIGHWAY   14
#define DEMO_H100_HWY_0      16
#define DEMO_H100_HWY_1      17
// ....
#define DEMO_H100_HWY_31     47


typedef struct DemoEnvT {
  char           *logFileName;
  OtxDataFormatE  law;
} DemoEnvT;


/*******************************************************************************
 Function Prototypes
*******************************************************************************/
static void DemoCreateEventQueue(void);
static void DemoOpenPhysicalDevices(OTX_INT32 iBrd);
static void DemoConnectDevices(OTX_INT32 iBrd);
static void DemoEnablePhysicalDevices(OTX_INT32 iBrd);
static void DemoPrintEvent(IN OtxEventDataS *pEventData);
static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT otxRc);
static void DemoLogf(char *formatStr, ...);
static void DemoHelpPanel(void);
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP);


/*******************************************************************************
 Global Data
*******************************************************************************/
OTX_EVENT   g_hNotificationEvent = OTX_INVALID_EVENT_VALUE;
OTX_HANDLE  g_hEventQueue  = 0;
FILE       *g_pDemoLogFile = NULL;
DemoEnvT    g_demoEnv;

struct {
  OTX_HANDLE hBoardController;
  OTX_HANDLE hTss;
  OTX_HANDLE hCodec[DEMO_MAX_CODECS];
} g_hPhysDevs[DEMO_NO_OF_BRDS];


struct {
  OTX_HANDLE hHs3MicPipe;
  OTX_HANDLE hHs3SpkrPipe;
  OTX_HANDLE hHs2MicPipe;
  OTX_HANDLE hHs2SpkrPipe;
  OTX_HANDLE hHs1MicPipe;
  OTX_HANDLE hHs1SpkrPipe;
  OTX_HANDLE hHs0MicPipe;
  OTX_HANDLE hHs0SpkrPipe;
} g_hPipes[DEMO_NO_OF_BRDS];


/*******************************************************************************
*******************************************************************************/


/*******************************************************************************

  @func

  Create the event queue to be used by the driver to inform the application
  about events, such as line status changes, or detected DMTF digits.

*******************************************************************************/
void DemoCreateEventQueue(void)
{
  OTX_RESULT result;

  // Create a notification Event
  result = OtxDrvCreateEvent(&g_hNotificationEvent);
  if (result == OTX_E_NOTIMPL) {
    g_hNotificationEvent = NULL;    // D16V86 apps cannot use event objects
    result = OTX_S_OK;
  }

  DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);

  // Create Event Queue for driver notifications
  result = OtxDrvCreateEventQueue(128, g_hNotificationEvent, &g_hEventQueue); DemoCheckRc(g_hEventQueue, result);

  // Enable Event Queue
  result = OtxDrvEnable(g_hEventQueue, OTX_TASK_SYNC,0); DemoCheckRc(g_hEventQueue, result);
}



/*******************************************************************************

  @func
    Open the Physical Devices on the Vidar-5x16-PCI board

  @comm
    Calls function <f OtxDrvCreatePhysicalDevice> returning a handle to a
    Device Object representing a physical device, which can be the
    entire board or an integrated circuit (chip) on the board. The number of
    physical devices is set per board and cannot change dynamically.

*******************************************************************************/
void DemoOpenPhysicalDevices(OTX_INT32 iBrd)
{
  OTX_RESULT result;
  OTX_INT32  i;

  // Connect to the Physical Thor-2 Board Controller
  DemoLogf("Open OTX_DEVICE_VIDAR_5x_PCI (Vidar-5x16-PCI board %d)\n", iBrd);
  result = OtxDrvOpenPhysicalDevice(0, OTX_DEVICE_VIDAR_5x_PCI, iBrd, iBrd, g_hEventQueue, &g_hPhysDevs[iBrd].hBoardController); DemoCheckRc(g_hPhysDevs[iBrd].hBoardController, result);

  // Get a handle to the Time-Space Switch (TSS) on the Vidar-5x16-PCI board
  DemoLogf("Open OTX_DEVICE_TSS\n");
  result = OtxDrvOpenPhysicalDevice(g_hPhysDevs[iBrd].hBoardController, OTX_DEVICE_TSS, 0, 0, g_hEventQueue, &g_hPhysDevs[iBrd].hTss); DemoCheckRc(g_hPhysDevs[iBrd].hBoardController, result);
  DemoLogf("Open OTX_DEVICE_TSS\n");
  result = OtxDrvEnable(g_hPhysDevs[iBrd].hTss, OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs[iBrd].hTss, result);

  // Get handles to Codec devices #0 and #1
  for (i = 0; i < 2; i++) {
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs[iBrd].hBoardController, OTX_DEVICE_CODEC, i, i, g_hEventQueue, &g_hPhysDevs[iBrd].hCodec[i]); DemoCheckRc(g_hPhysDevs[iBrd].hBoardController, result);
  }

}


/*******************************************************************************

  @func
    Enable Configured Physical Devices

*******************************************************************************/
void DemoEnablePhysicalDevices(OTX_INT32 iBrd)
{
  OTX_RESULT result;

  // Enable Physical Devices
  result = OtxDrvEnable(g_hPhysDevs[iBrd].hBoardController,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs[iBrd].hBoardController, result);

}







/*******************************************************************************

  @func
    Create the needed Pipe Devices and connect the physical and logical devices
    to pipes.

  @comm
    A Pipe Device is a logical entity representing a serial bit-stream (a subset of a
    Highway) with a data rate between 8 kbit/s and 2.048Mbit/s. A pipe is a characterized
    bu its capacity (in 8 kbit/s increments). The user can create a pipe with
    the driver function <f OtxDrvCreatePipe>. A pipe is connected to physical or
    logical devices and mapped to specific time-slots with the driver function
    <f OtxDrvConnectPipe>.

*******************************************************************************/
void DemoConnectDevices(
  OTX_INT32 iBrd
)
{
  OTX_RESULT result;

  // Create voice pipes for Hs2 and H3 (to be connected over the CT bus)
  result = OtxDrvCreatePipe(64, &g_hPipes[iBrd].hHs3MicPipe); DemoCheckRc(g_hPipes[iBrd].hHs3MicPipe, result);
  result = OtxDrvCreatePipe(64, &g_hPipes[iBrd].hHs3SpkrPipe); DemoCheckRc(g_hPipes[iBrd].hHs3SpkrPipe, result);
  result = OtxDrvCreatePipe(64, &g_hPipes[iBrd].hHs2MicPipe); DemoCheckRc(g_hPipes[iBrd].hHs2MicPipe, result);
  result = OtxDrvCreatePipe(64, &g_hPipes[iBrd].hHs2SpkrPipe); DemoCheckRc(g_hPipes[iBrd].hHs2SpkrPipe, result);

  // Create voice pipes for Hs0 and Hs1
  result = OtxDrvCreatePipe(64, &g_hPipes[iBrd].hHs1MicPipe); DemoCheckRc(g_hPipes[iBrd].hHs1MicPipe, result);
  result = OtxDrvCreatePipe(64, &g_hPipes[iBrd].hHs0MicPipe); DemoCheckRc(g_hPipes[iBrd].hHs0MicPipe, result);

  // Connect Hs0 <--> Hs1 in the time-space switch
  result = OtxDrvConnectPipe(g_hPipes[iBrd].hHs0MicPipe, OTX_PIPE_INPUT,  g_hPhysDevs[iBrd].hCodec[0], -1, NULL, 0); DemoCheckRc(g_hPipes[iBrd].hHs0MicPipe, result);
  result = OtxDrvConnectPipe(g_hPipes[iBrd].hHs0MicPipe, OTX_PIPE_OUTPUT, g_hPhysDevs[iBrd].hCodec[1], -1, NULL, 0); DemoCheckRc(g_hPipes[iBrd].hHs0MicPipe, result);

  result = OtxDrvConnectPipe(g_hPipes[iBrd].hHs1MicPipe, OTX_PIPE_INPUT,  g_hPhysDevs[iBrd].hCodec[1], -1, NULL, 0); DemoCheckRc(g_hPipes[iBrd].hHs1MicPipe, result);
  result = OtxDrvConnectPipe(g_hPipes[iBrd].hHs1MicPipe, OTX_PIPE_OUTPUT, g_hPhysDevs[iBrd].hCodec[0], -1, NULL, 0); DemoCheckRc(g_hPipes[iBrd].hHs1MicPipe, result);

}



static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT result)
/*******************************************************************************
  @func
    Check the result from API call. If the result is a failure code then prompt
    the user if he wants to exit.
*******************************************************************************/
{
  short c;
  if ( OTX_FAILURE(result) ) {

    if (hDevice == OTX_INVALID_HANDLE_VALUE) {
      DemoLogf("ERROR(%#lX) \n", result);
    } else {
      OTX_RESULT otxRc;
      char msgStr[256];
      OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
      OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
      OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
      OTX_UINT32      eDeviceType = 999;    // Device type for hDevice 
      OTX_UINT32      nDeviceNumber = 999;  // Device number for hDevice
      
      // Get the Device Type for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_TYPE, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_TYPE, &eDeviceType); 
      } 
      
      // Get the Device Number for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_NO, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_NO, &nDeviceNumber); 
      }
      
      // Convert the result code to a more descriptive string
      otxRc = OtxDrvResultCode2String(hDevice, result, 256, &msgStr[0]);
      if (OTX_SUCCESS(otxRc)) {
        DemoLogf("ERROR(%#lX) Dev(%X,%d,%d): %s \n", result, hDevice, (int) eDeviceType, (int)nDeviceNumber, msgStr);
      } else {
        DemoLogf("ERROR(%#lX) Dev(%X,%d,%d) Cannot translate this result code \n", result, hDevice, (int) eDeviceType, (int)nDeviceNumber);
      }
    }


    DemoLogf("Do you want to exit (Y/N) ");
    c = getch();
    if (c == 'y' || c == 'Y') {
      // Clean up
      OtxDrvReleaseLib();
      OsaConsoleExit();
			exit(1);
    }

  }
}



/*******************************************************************************
  @func
    Idle Function that check for Events from the OTX Driver
*******************************************************************************/
void *DemoIdleFunction(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;

  while (1) {
    OtxDrvWaitForSingleEvent(g_hNotificationEvent, OTX_TIME_INFINITY);
    //DemoLogf("Event signalled\n");
    do {
      result = OtxDrvGetEventData(g_hEventQueue, &eventData); DemoCheckRc(g_hEventQueue, result);

      if (result == OTX_S_OK) {
        // We have a driver event
        // Printf the event data.
        // In a real application branch based on the data
        DemoPrintEvent(&eventData);

      }
    } while (result == OTX_S_OK);
  }

  return 0;

}



/*******************************************************************************
  @func
    Prints a Driver Event. If HDLC Data, reads the data.
*******************************************************************************/
void DemoPrintEvent(
  OtxEventDataS *pEventData
)
{
  switch (pEventData->m_eDeviceType) {
  default:
    // Unhandled event
    DemoLogf("Event from an unknown object 0x%X\n", pEventData->m_hDevice);
    break;
  }


}

/*******************************************************************************

  @func

  Processes User Commands

  @comm

*******************************************************************************/
OTX_INT32 DemoProcessUserCommand(char commandChar)
{
  OTX_RESULT result;
  static OTX_INT32      nCurrentBrd = 0;

  switch (commandChar) {

  case '?':
  case 'h':
  case 'H':
    DemoLogf("Cmd  Synopsis\n");
    DemoLogf("===  ========\n");
    DemoLogf(" ?   Display Help text\n");
    DemoLogf(" h   Ditto\n");
    DemoLogf(" q   Quit\n");
    DemoLogf(" x   Connect HS2 to H.100 Highway 0, (TS0 and TS1)\n");
    DemoLogf(" X   Disconnect HS2.\n");
    DemoLogf(" c   Set Clock Source for current Board\n");
    DemoLogf(" 0-1 Select active Board, this Board is then used for subsequent commands\n");
    break;

  case 'x':
    {

      OTX_UINT8 sqTimeSlotMask[32];
      OTX_INT32 ts;
      OTX_INT32 iBrd;

      for (iBrd = 0; iBrd < DEMO_NO_OF_BRDS; iBrd++) {
        
        DemoLogf("Board#%d: Connecting HS2 to H100 Hwy0, ", iBrd);
        ts = 3; // HS0 is TS1, HS1 is TS2, HS2 is TS3, HS3 is TS4 on the DEMO_CODEC_HIGHWAY
        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        sqTimeSlotMask[ts] = 0xFF;
        result = OtxDrvConnectPipe(g_hPipes[iBrd].hHs2MicPipe, OTX_PIPE_INPUT,   g_hPhysDevs[iBrd].hTss, DEMO_CODEC_HIGHWAY, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes[iBrd].hHs2MicPipe, result);



        if (iBrd == 0) {
          ts = 0;
        } else {
          ts = 1;
        }
        DemoLogf("Mic to TS%d, ", ts);
        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        sqTimeSlotMask[ts] = 0xFF;
        result = OtxDrvConnectPipe(g_hPipes[iBrd].hHs2MicPipe, OTX_PIPE_OUTPUT,   g_hPhysDevs[iBrd].hTss, DEMO_H100_HWY_0, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes[iBrd].hHs2MicPipe, result);
        
        if (iBrd == 0) {
          ts = 1;
        } else {
          ts = 0;
        }
        DemoLogf("Spkr to TS%d \n", ts);
        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        sqTimeSlotMask[ts] = 0xFF;
        result = OtxDrvConnectPipe(g_hPipes[iBrd].hHs2SpkrPipe, OTX_PIPE_INPUT,   g_hPhysDevs[iBrd].hTss, DEMO_H100_HWY_0, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes[iBrd].hHs2SpkrPipe, result);
        
        ts = 3;
        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        sqTimeSlotMask[ts] = 0xFF;
        result = OtxDrvConnectPipe(g_hPipes[iBrd].hHs2SpkrPipe, OTX_PIPE_OUTPUT,   g_hPhysDevs[iBrd].hTss, DEMO_CODEC_HIGHWAY, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes[iBrd].hHs2SpkrPipe, result);
      }

      DemoLogf("Done.\n");
    }
    break;

  case 'X':
    {
      OTX_INT32 iBrd;

      for (iBrd = 0; iBrd < DEMO_NO_OF_BRDS; iBrd++) {
        DemoLogf("Board#%d: Disconnecting HS2 Pipes (Spkr and Mic)\n", iBrd);
        result = OtxDrvDisconnectPipe(g_hPipes[iBrd].hHs2MicPipe, OTX_PIPE_INPUT); DemoCheckRc(g_hPipes[iBrd].hHs2MicPipe, result);
        result = OtxDrvDisconnectPipe(g_hPipes[iBrd].hHs2MicPipe, OTX_PIPE_OUTPUT); DemoCheckRc(g_hPipes[iBrd].hHs2MicPipe, result);
        result = OtxDrvDisconnectPipe(g_hPipes[iBrd].hHs2SpkrPipe, OTX_PIPE_INPUT); DemoCheckRc(g_hPipes[iBrd].hHs2SpkrPipe, result);
        result = OtxDrvDisconnectPipe(g_hPipes[iBrd].hHs2SpkrPipe, OTX_PIPE_OUTPUT); DemoCheckRc(g_hPipes[iBrd].hHs2SpkrPipe, result);
      }
      
      DemoLogf("Done.\n");
    }
    break;


  case 'c':
    {
      OtxBrdClockSourceE  eClkSource;
      OtxBrdClockModeE    eClkMode;
      DemoLogf("Set Clock Source for Board#%d:\n", nCurrentBrd);
      DemoLogf("0 = OTX_CLOCK_SOURCE_INTERNAL  - LIs synchronized to on-board oscillator\n");
      DemoLogf("1 = OTX_CLOCK_SOURCE_LOCAL_0   - LIs synchronized to FA0 receive span\n");
      DemoLogf("2 = OTX_CLOCK_SOURCE_LOCAL_1   - LIs synchronized to FA1 receive span\n");
      DemoLogf("3 = OTX_CLOCK_SOURCE_A_CLOCKS  - H.100 Slave\n");

      switch (getch() - '0') {
      case 0:
        eClkSource = OTX_CLOCK_SOURCE_INTERNAL;
        eClkMode = OTX_CLOCK_MODE_H100;
        break;
      case 1:
        eClkSource = OTX_CLOCK_SOURCE_LOCAL_0;
        eClkMode = OTX_CLOCK_MODE_H100;
        break;
      case 2:
        eClkSource = OTX_CLOCK_SOURCE_LOCAL_1;
        eClkMode = OTX_CLOCK_MODE_H100;
        break;
      case 3:
        eClkSource = OTX_CLOCK_SOURCE_A_CLOCKS;
        eClkMode = OTX_CLOCK_MODE_H100;
        break;
      default:
        DemoLogf("Invalid entry. Setting Clock Source to OTX_CLOCK_SOURCE_INTERNAL\n");
        eClkSource = OTX_CLOCK_SOURCE_INTERNAL;
        eClkMode = OTX_CLOCK_MODE_LOCAL;
        break;
      }

      result = OtxBrdSetClocks(g_hPhysDevs[nCurrentBrd].hBoardController, eClkMode, eClkSource); DemoCheckRc(g_hPhysDevs[nCurrentBrd].hBoardController, result);
      DemoLogf("Done.\n");
    }
    break;
  
  case '0':
  case '1':
    // Select current Board 
    nCurrentBrd = commandChar - '0';
    DemoLogf("Brd#%d Selected ....\n", nCurrentBrd);
    break;

  case 'q':
  case 'Q':
    return 1;
    break;

  default:
    DemoLogf("Unknown command. Press '?' for help, 'q' to quit\n");
    break;
  }

  return 0;
}



/*******************************************************************************

  @func

  Parse the program's command line arguments and calls upon
  requested actions.

  There are some optional switches that can be used following
  the driver name on the command line. Switches are are
  separated by spaces, and can appear in arbitrary order.
  Switches consist of a hyphen and a letter:

  @comm

*******************************************************************************/
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP) {
  short  i;

  // Set reasonable defaults before reading from command line
  envP->logFileName  = DEMO_DEFAULT_LOG_FILE;
  envP->law          = OTX_DATA_FORMAT_COMPAND8_U_LAW;

  for (i = 1; i < argc; i++) {        // Scan command line arguments
    OsaStdStrUpr(argv[i]);
    if (*argv[i] == '-' || *argv[i] == '/') {  // Is argument a legal switch?
      if (*(argv[i]+1) == 'L') {   // Log file
        envP->logFileName = argv[i]+2;
      } else if (*(argv[i]+1) == 'C') {   // Companding Law
        if (*(argv[i]+2) == 'A') {   // A-law
          envP->law = OTX_DATA_FORMAT_COMPAND8_A_LAW;
        } else if (*(argv[i]+2) == 'U') {   // U-law
          envP->law = OTX_DATA_FORMAT_COMPAND8_U_LAW;
        } else {
          DemoLogf("Error: Invalid companding law command line switch %s\n", argv[i]);
          DemoHelpPanel();
          OsaConsoleExit();
					exit(1);
        }
      } else {
        DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
        DemoHelpPanel();
        OsaConsoleExit();
				exit(1);
      }
    } else {
      DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
      DemoHelpPanel();
      OsaConsoleExit();
			exit(1);
    }
  }

}


/*******************************************************************************

  @func

  Displays a help for the command line.

  @comm

*******************************************************************************/
static void DemoHelpPanel()
{
  DemoLogf("\nCommand line switches:\n");
  DemoLogf(" /L<fileName> : Log file name\n");
  DemoLogf(" /CA          : A-law\n");
  DemoLogf(" /CU          : u-law\n");
}


/*******************************************************************************

  @func

  Displays a message on the screen and prints it to the logfile.

  @comm

*******************************************************************************/
static void DemoLogf(char *formatStr, ...)
{
  va_list argList;
  va_start(argList, formatStr);

  OsaConsolePrintF(formatStr, argList);
  if (g_pDemoLogFile) vfprintf(g_pDemoLogFile,   formatStr, argList);

  va_end(argList);

}



/*******************************************************************************
*******************************************************************************/
int main(int argc, char *argv[])
{
  OTX_INT32 exitFlag = 0;
  OTX_RESULT result;  // OTX Standard Return Code
  char commandChar;
  OTX_INT32 iBrd;

  DemoLogf("%s, %s, Odin TeleSystems Inc.\n", argv[0], __DATE__);
  DemoLogf("2 x Vidar-5x16-PCI Example program.\n");
  DemoParseCmdLineArgs(argc, argv, &g_demoEnv);

  if ((g_pDemoLogFile = fopen(g_demoEnv.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", g_demoEnv.logFileName);
    OsaConsoleExit();
		exit(-1);
  }

  DemoLogf("Log file is  %s\n", g_demoEnv.logFileName);
  DemoLogf("\nPress 'h' for help\n");

  // Connect to the Driver
  DemoLogf("Connecting lib...\n");
  result = OtxDrvConnectLib(); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);

  // Create Event Queues for Driver --> Application Communications
  DemoLogf("Creating event queue...\n");
  DemoCreateEventQueue();

  for (iBrd = 0; iBrd < DEMO_NO_OF_BRDS; iBrd++) {
    
    // Connect (get handles) to the physical devices on the Vidar-5x16-PCI board
    DemoLogf("Opening physical devices...\n");
    DemoOpenPhysicalDevices(iBrd);

  }
    
  // Set clocks sources for the two boards
  result = OtxBrdSetClocks(g_hPhysDevs[0].hBoardController, OTX_CLOCK_MODE_H100, OTX_CLOCK_SOURCE_INTERNAL); DemoCheckRc(g_hPhysDevs[0].hBoardController, result);
  result = OtxBrdSetClocks(g_hPhysDevs[1].hBoardController, OTX_CLOCK_MODE_H100,  OTX_CLOCK_SOURCE_A_CLOCKS); DemoCheckRc(g_hPhysDevs[1].hBoardController, result);

  for (iBrd = 0; iBrd < DEMO_NO_OF_BRDS; iBrd++) {

    // Enable physical devices. Their device state should now be OTX_DEVICE_WORKING
    DemoEnablePhysicalDevices(iBrd);

    // Create and get handles to the Pipe devices needed by this application
    DemoConnectDevices(iBrd);

  }


  // Read Serial Number from the boards
  {
    OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
    OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
    OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
    char *pszSerialNumber;

    for (iBrd = 0; iBrd < DEMO_NO_OF_BRDS; iBrd++) {
      result = OtxDrvGetAttributeType(g_hPhysDevs[iBrd].hBoardController, OTX_ATTR_BRD_SERIAL_NUMBER, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(g_hPhysDevs[iBrd].hBoardController, result);
      assert(eAttribType == OTX_ATTRIB_SZ);
      assert(bReadOnly == OTX_TRUE);
      pszSerialNumber = (char *)malloc(cbAttribValue * sizeof(char));
      result = OtxDrvGetAttributeValueSz(g_hPhysDevs[iBrd].hBoardController, OTX_ATTR_BRD_SERIAL_NUMBER, pszSerialNumber, cbAttribValue); DemoCheckRc(g_hPhysDevs[iBrd].hBoardController, result);
      DemoLogf("Board#%d Serial Number: %s\n", iBrd, pszSerialNumber);
      free(pszSerialNumber);
    }
  }

  // Main Loop. Loop here and wait for Events from the driver or commands from the user ....
	OsaThreadStart(DemoIdleFunction);

  while (!exitFlag) {

		commandChar = getch();
    exitFlag = DemoProcessUserCommand(commandChar);

  }

  // Clean up
  OtxDrvReleaseLib();
	OsaConsoleExit();

  return 0;
}

